#################################################################################################
##
##   TkBib  
##
##  Ascii-based bibliography database system written in Tcl/TK 
##  
##  Copyright (C) 1997 Daniel Strite
##  This program is free software and is governed by the GNU General Public Licence as published
##  by the Free Software Foundation, version 2 or later.  See the GNU license for details on
##  merchantibility, warranty, and fitness for a particular purpose of this program.
##
##
#################################################################################################

#!/bin/sh
# the next line restarts using wish\
exec wish "$0" "$@"


#include <tk.h>


wm minsize . 40 25
wm title . "TkBib  Copyright 1997 Daniel Strite (GNU GPL)"

# Create three frames for the buttons & load file (upper) and query terms (lower)
frame .but -borderwidth 10
frame .en -borderwidth 10
frame .imp -borderwidth 10
pack .but .en .imp -side top -fill x

# Set default file to use in $biblio
set biblio biblio.txt

# Create the buttons including the SaveAs entry & label
button .en.doit -text DoIt -command Doswitch
button .but.quit -text Quit -command exit
button .but.saveit -text SaveIt -command Saveit

label .but.1 -text "Current file:" -padx 0
entry .but.biblio -width 15 -relief sunken -textvariable biblio 
label .but.2 -text "Save query as:" -padx 0
entry .but.saveas -width 15 -relief sunken -textvariable saveas 

label .imp.1 -text "BRS file:" -padx 0
entry .imp.filesel -width 15 -relief sunken -textvariable brsfile
button .imp.import -text ImportBRS -command Import
button .imp.append -text AppendBRS -command Appendbrs
button .imp.edit -text "Edit/Add Entries" -command Tkedit
# button .imp.savebuf -text "Save Buffer" -command Savebuf

pack  .but.1 .but.biblio -side left 
pack .but.2 .but.saveas .but.saveit  -side left
pack .but.quit -side right
pack .imp.filesel .imp.1 .imp.import .imp.append -side right 
pack .imp.edit -side left
# pack .imp.savebuf
# Set up the radiobuttons for the conditional option on the entry frame
checkbutton .en.or -text "Terms Conditional?" -variable orvar
pack .en.or -side left

# Set up entry widgets and labels for entering query terms
label .en.1 -text Terms: -padx 0
entry .en.trm1 -width 20 -relief sunken -textvariable search1
entry .en.trm2 -width 20 -relief sunken -textvariable search2
entry .en.trm3 -width 20 -relief sunken -textvariable search3
pack .en.1  -side left
pack .en.trm1 .en.trm2 .en.trm3  .en.doit -side left -fill x -expand true


#  Creates a "Loading database" message while the larghe biblio.txt file is being loaded
#  because it takes several seconds to load


proc WaitLoading {} {
	after idle {.dialog1.msg configure -wraplength 0i}
	set i [tk_dialog .dialog1 "TkBib" {Please wait...} info 0 OK Cancel]
	switch $i {
    		0 {puts "Go For It!"}
    		1 {puts "Ooops!!"}
	}
}

# Procedure for displaying current file with scrollbars.  Default is Biblio.txt
# From Welsh (1995).

proc ScrolledText { f width height } {
	frame $f
	# The setgrid setting allows the window to be resized.
	text $f.text -width $width -height $height \
		-setgrid true -wrap word \
		-xscrollcommand [list $f.xscroll set] \
		-yscrollcommand [list $f.yscroll set]
	scrollbar $f.xscroll -orient horizontal \
		-command [list $f.text xview]
	scrollbar $f.yscroll -orient vertical \
		-command [list $f.text yview]
	pack $f.xscroll -side bottom -fill x
	pack $f.yscroll -side right -fill y
	# The fill and expand are needed when resizing.
	pack $f.text -side left -fill both -expand true
	pack $f -side top -fill both -expand true
	return $f.text
}


# Create a read-only text widget with scrollbars using the default
# database.  From Welsh (1995).

proc ScrolledTextCreate {} {
    global home biblio
    catch {destroy .f}
    set t [ScrolledText .f 80 30]
    set in [open $home\\tcl\\$biblio]
    $t insert end [read $in]
    close $in

}

# Update the test widget with "splyt" which is the final output file
# for the Format and Import procedures

proc ScrolledTextUpdate {} {
    global home 
 
    catch {destroy .f}
    set t [ScrolledText .f 80 30]
    set in [open $home\\tcl\\splyt]
    $t insert end [read $in]
    close $in
}


# THIS IS NOT FUNCTIONAL.  MAY USE AN EXTERNAL EDITOR 
# THAN TRY TO EMBED ONE.  MUCH EASIER (e.g.  TkEdit procedure)
#
# Save the current buffer 

#proc Savebuf {} {
#    global home
#    set out [open $home/buf w+]	    
# 
#    close $out
#}


# Switch to determine whether conditional
# or summative query is used (Doit vs Doitmore)

proc Doswitch {} {
      global orvar
      switch $orvar {
      0 {Doitmore}
      1 {Doit}
  }
}   


# Query procedure using Tcl regexp command for greater portability

proc Doit {} {
    global biblio search1 search2 search3 home
    set d [open $home\\tcl\\$biblio r]
    set out [open $home\\tcl\\unformat w+]	    
    set pat $search1
    set pat2 $search2
    set pat3 $search3 
    while {[gets $d line] >= 0} {
	if [regexp -nocase $pat $line]&[regexp -nocase $pat2 $line]&[regexp -nocase $pat3 $line] {
	    puts $out $line
	}
    }
    close $d
    close $out
    Filter
    ScrolledTextUpdate
}


# Query proc same as Doit except it is summative and there are three different 
# query formulas depending  on the number of terms entered.

proc Doitmore {} {
    global biblio search1 search2 search3 home
    set d [open $home\\tcl\\$biblio r]
    set out [open $home\\tcl\\unformat w+]	    
    set pat $search1
    set pat2 $search2
    set pat3 $search3 
    if {[string length $pat3] > 0} {
	while {[gets $d line] >= 0} {
	    if {[regexp -nocase $pat|$pat2|$pat3 $line]} { 
	              puts $out $line
	    }
	}
    } elseif {[string length $pat2] > 0} {
	while {[gets $d line] >= 0} {
	    if {[regexp -nocase $pat|$pat2 $line]} { 
	              puts $out $line
	    }
	}
    } else {
    	while {[gets $d line] >= 0} {
	    if {[regexp -nocase $pat|$pat2 $line]} { 
			  puts $out $line
	    }
	}
    }
    close $d
    close $out
    Filter
    ScrolledTextUpdate
}



# Save the most recent query (formatted) as textfile  

proc Saveit {} { 
    global saveas home     
    set d [open $home\\tcl\\splyt r] 
    set out [open $home\\tcl\\$saveas.txt w+]	     
    while {[gets $d line] >= 0} { 
	    puts $out $line
    }
    close $d
    close $out
}


# Gets rid of the % field delimiter and the extra fields (key variables, comments, location)
# The "splyt" file produced is then viewed and can be saved. 

proc Filter {} {
    global home
    set d [open $home\\tcl\\unformat r]
    set splyt [open $home\\tcl\\splyt w+]
    while {[gets $d line] >= 0} {
	puts $splyt [join [lrange [split $line %] 0 2 ]  ]
    }
    close $d
    close $splyt
}

# Imports Medline BRS format files from a file specified
# in the variable $brsfile.
# Returns the reformated file into the test widget for viewing. 

proc Import {} {
    global home brsfile subin
    set subin "%"
    set d [open $home\\tcl\\$brsfile r]
    set brs [open $home\\tcl\\imp w+]
    while {[gets $d line] >= 0} {
	regsub -all {AU|TI|SO} $line $subin x
	puts -nonewline $brs $x
    }
    close $d
    close $brs

    set newline \\\n
    set d [open $home\\tcl\\imp r]
    set brs [open $home\\tcl\\imp2 w+]
    while {[gets $d line] >= 0} {
	regsub -all {<.>} $line $newline y
	puts $brs $y	
    }
    close $d
    close $brs

    set sub " " 
    set d [open $home\\tcl\\imp2 r]
    set brs [open $home\\tcl\\splyt w+]
    while {[gets $d line] >= 0} {
	regsub -all {^AN} $line $sub z
	puts $brs [join [string trim [lrange [split $z ] 3 end] \\] ]
    }
    close $d
    close $brs

    ScrolledTextUpdate    
}

proc Appendbrs {} {   
    global home biblio brsfile 
    set bib [open $home\\tcl\\$biblio a]
    set brs [open $home\\tcl\\splyt r]
    while {[gets $brs line] >= 0} {
	puts $bib $line
    }
    close $bib
    close $brs
    ScrolledTextCreate
}


# Run TkEdit (or Word) for extensive browsing and also as the easy way to add entries
proc Tkedit {} {
      global home biblio
    set default tcl
    set d [exec c:\\msoffice\\winword\\winword.exe /a \\$default\\$biblio]
}




######################
#  DEFAULTS          #
######################

# Set the default directory and the default file
set home c:\\

# Set the conditional variable default to non-conditional (Doitmore is then the
# default query)
set orvar 1

# Set sample or default BRS file name
set brsfile brs.txt

# Global bindings
bind all <Control-c> {destroy %W}
bind all <Enter> {focus %W}




## Create the text box showing the current file.  
ScrolledTextCreate



